"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = require("axios");
const common_1 = require("../common");
const system_1 = require("../system");
const Utils_1 = require("../system/Utils");
const Consts_1 = require("./Consts");
let lastAuthHeaders = -1;
let cachedAuthHeaders = {};
function getAuthHeaders(useCachedHeaders = true) {
    return __awaiter(this, void 0, void 0, function* () {
        if (process.env.NODE_ENV === 'test') {
            return {};
        }
        const timestamp = Date.now();
        if (useCachedHeaders) {
            if (timestamp - lastAuthHeaders < 300000) {
                return cachedAuthHeaders;
            }
        }
        lastAuthHeaders = timestamp;
        const auth = system_1.default.getLibModule(system_1.LibModule.AUTH);
        const organization = yield auth.getOrganization();
        cachedAuthHeaders = {
            [Consts_1.X_ORGANIZATION]: organization.module_id,
            [Consts_1.X_CLIENT_TZ]: Intl.DateTimeFormat().resolvedOptions().timeZone,
            'Content-Type': 'application/json;charset=UTF-8',
        };
        return cachedAuthHeaders;
    });
}
function rejectedBecauseAuth(e) {
    return Boolean(e.response && e.response.status && (e.response.status === Consts_1.FORBIDDEN || e.response.status === Consts_1.UNAUTHORIZED));
}
/**
 * Makes sure that a URI contains no leading '/' characters
 *
 * @param uri The URI being sanitized
 */
function sanitizeUri(uri) {
    const length = uri.length;
    let i = 0;
    for (; i < length; i++) {
        if (uri.charAt(i) !== '/') {
            break;
        }
    }
    if (i === 0) {
        return uri;
    }
    return uri.substr(i);
}
function request(method, uri, data, headers, canRetryAuth = true) {
    return __awaiter(this, void 0, void 0, function* () {
        let authFailed = false;
        const requestSettings = {
            method,
            withCredentials: true,
        };
        if (data) {
            if (method === Consts_1.GET || method === Consts_1.DELETE) {
                requestSettings.params = data;
            }
            else {
                requestSettings.data = data;
            }
        }
        const authHeaders = yield getAuthHeaders(canRetryAuth);
        requestSettings.headers = Object.assign(Object.assign(Object.assign({}, authHeaders), system_1.default.getHttpHeaders()), headers);
        try {
            const url = `${system_1.default.nexus.getUrl()}/${sanitizeUri(uri)}`;
            const response = (yield axios_1.default(url, requestSettings));
            if (!response || !response.data) {
                return null;
            }
            return response.data;
        }
        catch (e) {
            authFailed = rejectedBecauseAuth(e);
        }
        if (authFailed && canRetryAuth) {
            // TODO: consider setting an amount of times to retry auth
            const user = yield system_1.default.nexus.reconnect();
            if (user) {
                return request(method, uri, data, headers, false);
            }
        }
        return null;
    });
}
function get(uri, params, headers) {
    return request(Consts_1.GET, uri, params, headers);
}
function del(uri, params, headers) {
    return request(Consts_1.DELETE, uri, params, headers);
}
function post(uri, payload, headers) {
    return request(Consts_1.POST, uri, payload, headers);
}
function put(uri, payload, headers) {
    return request(Consts_1.PUT, uri, payload, headers);
}
function search(domain, module, filters, options) {
    return __awaiter(this, void 0, void 0, function* () {
        common_1.validateDomainAndModule(domain, module);
        const domainUri = common_1.domainToUri(domain);
        const moduleUri = common_1.moduleToUri(module);
        const response = yield request(Consts_1.PUT, `${domainUri}/${moduleUri}/search`, Object.assign(Object.assign({}, options), { filters }), options === null || options === void 0 ? void 0 : options.headers);
        return response ? response : [];
    });
}
const api = Object.freeze(Utils_1.makeCallable(request, { get, delete: del, post, put, search }));
exports.default = system_1.default.sealModule(api);
